@@ -4,7 +4,6 @@ import os
 
 from twisted.trial import unittest
 
-from buildbot.twcompat import maybeWait
 from buildbot.sourcestamp import SourceStamp
 from buildbot.process import base
 from buildbot.steps.shell import ShellCommand, WithProperties
@@ -51,67 +50,83 @@ class Interpolate(unittest.TestCase):
     def testWithProperties(self):
         self.build.setProperty("revision", 47)
         self.failUnlessEqual(self.build_status.getProperty("revision"), 47)
-        c = ShellCommand(workdir=dir, build=self.build,
+        c = ShellCommand(workdir=dir,
                          command=["tar", "czf",
                                   WithProperties("build-%s.tar.gz",
                                                  "revision"),
                                   "source"])
+        c.setBuild(self.build)
         cmd = c._interpolateProperties(c.command)
         self.failUnlessEqual(cmd,
                              ["tar", "czf", "build-47.tar.gz", "source"])
 
+    def testWorkdir(self):
+        self.build.setProperty("revision", 47)
+        self.failUnlessEqual(self.build_status.getProperty("revision"), 47)
+        c = ShellCommand(command=["tar", "czf", "foo.tar.gz", "source"])
+        c.setBuild(self.build)
+        workdir = WithProperties("workdir-%d", "revision")
+        workdir = c._interpolateWorkdir(workdir)
+        self.failUnlessEqual(workdir, "workdir-47")
+
     def testWithPropertiesDict(self):
         self.build.setProperty("other", "foo")
         self.build.setProperty("missing", None)
-        c = ShellCommand(workdir=dir, build=self.build,
+        c = ShellCommand(workdir=dir,
                          command=["tar", "czf",
                                   WithProperties("build-%(other)s.tar.gz"),
                                   "source"])
+        c.setBuild(self.build)
         cmd = c._interpolateProperties(c.command)
         self.failUnlessEqual(cmd,
                              ["tar", "czf", "build-foo.tar.gz", "source"])
 
     def testWithPropertiesEmpty(self):
         self.build.setProperty("empty", None)
-        c = ShellCommand(workdir=dir, build=self.build,
+        c = ShellCommand(workdir=dir,
                          command=["tar", "czf",
                                   WithProperties("build-%(empty)s.tar.gz"),
                                   "source"])
+        c.setBuild(self.build)
         cmd = c._interpolateProperties(c.command)
         self.failUnlessEqual(cmd,
                              ["tar", "czf", "build-.tar.gz", "source"])
 
     def testCustomBuildStep(self):
-        c = MyBuildStep(workdir=dir, build=self.build)
+        c = MyBuildStep(workdir=dir)
+        c.setBuild(self.build)
         cmd = c._interpolateProperties(c.command)
         self.failUnlessEqual(cmd,
                              ["tar", "czf", "build-1234.tar.gz", "source"])
 
     def testSourceStamp(self):
-        c = ShellCommand(workdir=dir, build=self.build,
+        c = ShellCommand(workdir=dir,
                          command=["touch",
                                   WithProperties("%s-dir", "branch"),
                                   WithProperties("%s-rev", "revision"),
                                   ])
+        c.setBuild(self.build)
         cmd = c._interpolateProperties(c.command)
         self.failUnlessEqual(cmd,
                              ["touch", "branch2-dir", "1234-rev"])
 
     def testSlaveName(self):
-        c = ShellCommand(workdir=dir, build=self.build,
+        c = ShellCommand(workdir=dir,
                          command=["touch",
                                   WithProperties("%s-slave", "slavename"),
                                   ])
+        c.setBuild(self.build)
         cmd = c._interpolateProperties(c.command)
         self.failUnlessEqual(cmd,
                              ["touch", "bot12-slave"])
 
     def testBuildNumber(self):
-        c = ShellCommand(workdir=dir, build=self.build,
+        c = ShellCommand(workdir=dir,
                          command=["touch",
                                   WithProperties("build-%d", "buildnumber"),
                                   WithProperties("builder-%s", "buildername"),
                                   ])
+        c.setBuild(self.build)
         cmd = c._interpolateProperties(c.command)
         self.failUnlessEqual(cmd,
                              ["touch", "build-5", "builder-fakebuilder"])
@@ -120,11 +135,11 @@ class Interpolate(unittest.TestCase):
 run_config = """
 from buildbot.process import factory
 from buildbot.steps.shell import ShellCommand, WithProperties
+from buildbot.buildslave import BuildSlave
 s = factory.s
 
 BuildmasterConfig = c = {}
-c['bots'] = [('bot1', 'sekrit')]
-c['sources'] = []
+c['slaves'] = [BuildSlave('bot1', 'sekrit')]
 c['schedulers'] = []
 c['slavePortnum'] = 0
 
@@ -161,7 +176,7 @@ class Run(RunMixin, unittest.TestCase):
             self.failUnless(os.path.exists(f))
             return res
         d.addCallback(_check_touch)
-        return maybeWait(d)
+        return d
 
 
 # we test got_revision in test_vc
